#include "3dwindow.h"
#include <color.h>
#include "objects.h"

const CVector NullVector(0,0,0);
const CMatrix I(1,0,0,0,1,0,0,0,1);
const TColor COL_BACK(0,0,0);
const TColor COL_CROSSLASER(255,0,191);
const TColor COL_CROSSELECTRON(0,255,100);
const TColor COL_CROSSDIODE(255,0,191);
const TColor COL_REALCROSSDIODE(0,200,200);

//*** CLine ***
void CLine::Paint(TDC& dc) {
  TPen pen(color);
  dc.SetROP2(R2_XORPEN);
  dc.SelectObject(pen);
  dc.MoveTo(x1,y1);
  dc.LineTo(x2,y2);
}

//*** CPoint ***
void CPoint::Paint(TDC& dc) {
  TPen pen(color);
  dc.SetROP2(R2_XORPEN);
  dc.SelectObject(pen);
  dc.MoveTo(x-10,y);
  dc.LineTo(x+10,y);
  dc.MoveTo(x,y-10);
  dc.LineTo(x,y+10);
}

//*** C3DWindow ***
C3DWindow::C3DWindow(TWindow* parent,int x, int y, int w, int h,
	const CMatrix& amatrix, const CVector& atrans,const int adx,const int ady)
			 :TWindow(parent,0,0) {
  matrix=amatrix;
  trans=atrans;
  new TGroupBox(parent,0,"",x-1, y-1,w+2,h+2);
  Attr.X=x;
  Attr.Y=y;
  Attr.W=w;
  Attr.H=h;
  dx=adx;
  dy=ady;
  SetBkgndColor(COL_BACK);
}

void C3DWindow::Rotate(int axe,double phi) {
  CMatrix *mat;
  switch (axe) {
	 case 0:
		mat=new CRotationX(phi);
	 break;
	 case 1:
		mat=new CRotationY(phi);
	 break;
	 default:
		mat=new CRotationZ(phi);
	 break;
  }
  matrix=matrix*(*mat);
  delete mat;
}

void C3DWindow::Line3D(const CVector& a, const CVector& b,const TColor& color, int xor) {
  TClientDC dc(*this);
  if (xor) dc.SetROP2(R2_XORPEN);
  TPen pen(color);
  dc.SelectObject(pen);
  CVector h=matrix*(a-trans);
  dc.MoveTo((int)h[0]+dx,(int)h[1]+dy);
  h=matrix*(b-trans);
  dc.LineTo((int)h[0]+dx,(int)h[1]+dy);
}

void C3DWindow::Quadrat3D(const CVector& fix, const CVector& x, const CVector& y,const TColor& color) {
  CVector ul=fix-x*0.5-y*0.5;
  CVector h;
  Line3D(ul,h=ul+x,color);
  Line3D(h,ul=h+y,color);
  Line3D(ul,h=ul-x,color);
  Line3D(h,h-y,color);
}

void C3DWindow::Box3D(CVector& fix, CVector& x, CVector& y, CVector& z,const TColor& color) {
  Quadrat3D(fix-z*0.5,x,y,color);
  Quadrat3D(fix+z*0.5,x,y,color);
  CVector ul=fix-z*0.5-x*0.5-y*0.5;
  CVector ol=fix+z*0.5-x*0.5-y*0.5;
  CVector oh;
  CVector uh;
  Line3D(ol,ul,color);
  Line3D(oh=ol+x,uh=ul+x,color);
  Line3D(ol=oh+y,ul=uh+y,color);
  Line3D(ol-x,ul-x,color);
}

void C3DWindow::Point3D(const CVector& fix,const TColor& color) {
  TClientDC dc(*this);
  TPen pen(color);
  dc.SelectObject(pen);
  CVector h=matrix*(fix-trans);
  dc.Ellipse((int)h[0]-2+dx,(int)h[1]-2+dy,(int)h[0]+2+dx,(int)h[1]+2+dy);
}

void C3DWindow::FatLine3D(const CVector& a, const CVector& b,const CVector& size, const TColor& color) {
  TClientDC dc(*this);
  CVector h=matrix*size;
  TPen pen(color,abs(int(h[1])));
  dc.SelectObject(pen);
  h=matrix*(a-trans);
  dc.MoveTo((int)h[0]+dx,(int)h[1]+dy);
  h=matrix*(b-trans);
  dc.LineTo((int)h[0]+dx,(int)h[1]+dy);
}

//*** CDiodeWindow ***
CDiodeWindow::CDiodeWindow(CDiode *adiode, TWindow* parent,int x, int y,
  int w, int h, const CMatrix& amatrix, const CVector& atrans,const int adx,const int ady)
					 :C3DWindow(parent,x,y,w,h,amatrix,atrans,adx,ady)
					 ,laser(COL_CROSSDIODE)
					 ,realpos(COL_REALCROSSDIODE) {
  diode=adiode;
}

void CDiodeWindow::Paint(TDC& adc, BOOL a, TRect& arec) {
  C3DWindow::Paint(adc,a,arec);
  TClientDC dc(*this);
  TBrush brush(COL_BACK);
  dc.SelectObject(brush);
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  dc.FillRect(rec,brush);
  CVector pos=matrix*(diode->GetPos()-trans);
  laser.SetPoint((int)pos[0]+dx,(int)pos[1]+dy);
  laser.Paint(dc);
  pos=matrix*(diode->GetRealPos()-trans);
  realpos.SetPoint((int)pos[0]+dx,(int)pos[1]+dy);
  realpos.Paint(dc);
}

void CDiodeWindow::Update() {
  TClientDC dc(*this);
  dc.SetROP2(R2_XORPEN);
  laser.Paint(dc);
  CVector pos=matrix*(diode->GetPos()-trans);
  laser.SetPoint((int)pos[0]+dx,(int)pos[1]+dy);
  laser.Paint(dc);
  realpos.Paint(dc);
  pos=matrix*(diode->GetRealPos()-trans);
  realpos.SetPoint((int)pos[0]+dx,(int)pos[1]+dy);
  realpos.Paint(dc);
}

//*** CObject3DWindow***
CObject3DWindow::CObject3DWindow(C3DObject *aobject, TWindow* parent,int x, int y, int w, int h)
					 :C3DWindow(parent,x,y,w,h,I,NullVector) {
  object=aobject;
  amplify=0;
}

void CObject3DWindow::Paint(TDC& , BOOL , TRect&) {
  TClientDC dc(*this);
  TBrush brush(COL_BACK);
  dc.SelectObject(brush);
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  dc.FillRect(rec,brush);
  object->Paint(*this,amplify);
}

void CObject3DWindow::Update() {
  TClientDC dc(*this);
  TBrush brush(COL_BACK);
  dc.SelectObject(brush);
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  dc.FillRect(rec,brush);
  object->Paint(*this,amplify);
}

void CObject3DWindow::PaintNow() {
  object->Paint(*this,amplify);
}

void CObject3DWindow::SetAmplify(int aamplify) {
  amplify=aamplify;
  Update();
}

//*** CTerminator3DWindow ***
CTerminator3DWindow::CTerminator3DWindow(CTerminator* aterminator,
  TWindow* parent,int x, int y, int w, int h,
  const CMatrix& matrix, const CVector& trans)
						  :C3DWindow(parent,x,y,w,h,matrix,trans) {
  terminator=aterminator;
}

void CTerminator3DWindow::Paint(TDC& , BOOL, TRect& ) {
  TClientDC dc(*this);
  CVector a,b;
  TColor color;
  TBrush brush(COL_BACK);
  dc.SelectObject(brush);
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  dc.FillRect(rec,brush);
  terminator->PaintElectron(*this);
  terminator->GetLine(a,b,color);
  CVector begin=matrix*(a-trans);
  CVector end=matrix*(b-trans);
  line.SetLine((int)begin[0]+dx,(int)begin[1]+dy,(int)end[0]+dx,(int)end[1]+dy,color);
  line.Paint(dc);
}

void CTerminator3DWindow::Update() {
  CVector a,b;
  TColor color;
  TClientDC dc(*this);
  dc.SetROP2(R2_XORPEN);
  line.Paint(dc);
  terminator->GetLine(a,b,color);
  CVector begin=matrix*(a-trans);
  CVector end=matrix*(b-trans);
  line.SetLine((int)begin[0]+dx,(int)begin[1]+dy,(int)end[0]+dx,(int)end[1]+dy,color);
  line.Paint(dc);
}

void CTerminator3DWindow::NewSize() {
  CalcTransform();
  TClientDC dc(*this);
  BOOL a=0;
  TRect rec;
  Paint(dc,a,rec);
}

//*** CTerminatorYZWindow ***
CTerminatorYZWindow::CTerminatorYZWindow(CTerminator* aterminator,
  TWindow* parent,int ax, int ay, int aw, int ah)
  :CTerminator3DWindow(aterminator,parent,ax,ay,aw,ah,I,NullVector) {
}

void CTerminatorYZWindow::CalcTransform() {
  CVector fix;
  double x,y,z;
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  terminator->GetBox(fix,x,y,z);
  CMatrix matrix(0,0,rec.right/z,0,-rec.bottom/y,0,1,0,0);
  CVector trans(0,y/2,-z);
  SetTransform(matrix,fix+trans);
}

//*** CTerminatorXZWindow ***
CTerminatorXZWindow::CTerminatorXZWindow(CTerminator* aterminator,
  TWindow* parent,int ax, int ay, int aw, int ah)
  :CTerminator3DWindow(aterminator,parent,ax,ay,aw,ah,I,NullVector) {
};

void CTerminatorXZWindow::CalcTransform() {
  CVector fix;
  double x,y,z;
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  terminator->GetBox(fix,x,y,z);
  CMatrix matrix(0,0,rec.right/z,-rec.bottom/x,0,0,1,0,0);
  CVector trans(x/2,0,-z);
  SetTransform(matrix,fix+trans);
}

//*** CTerminatorYXWindow ***
CTerminatorYXWindow::CTerminatorYXWindow(CTerminator* aterminator, TWindow* parent,int ax, int ay, int aw, int ah)
						  :CTerminator3DWindow(aterminator,parent,ax,ay,aw,ah,I,NullVector)
						  ,laser(COL_CROSSLASER)
						  ,electron(COL_CROSSELECTRON) {
}

void CTerminatorYXWindow::CalcTransform() {
  CVector fix;
  double x,y,z;
  TRect rec;
  GetWindowRect(rec);
  rec.Offset(-rec.left,-rec.top);
  terminator->GetBox(fix,x,y,z);
  CMatrix matrix(rec.right/x,0,0,0,-rec.bottom/y,0,0,0,1);
  CVector trans(-x/2,y/2,-z);
  SetTransform(matrix,trans+fix);
}

void CTerminatorYXWindow::Paint(TDC& adc, BOOL a, TRect& rec) {
  CTerminator3DWindow::Paint(adc,a,rec);
  TClientDC dc(*this);
  CVector pl,pe;
  terminator->GetPoints(pl,pe);
  pl=matrix*(pl-trans);
  laser.SetPoint((int)pl[0]+dx,(int)pl[1]+dy);
  laser.Paint(dc);
  pe=matrix*(pe-trans);
  electron.SetPoint((int)pe[0]+dx,(int)pe[1]+dy);
  electron.Paint(dc);
}

void CTerminatorYXWindow::Update() {
  CTerminator3DWindow::Update();
  TClientDC dc(*this);
  dc.SetROP2(R2_XORPEN);
  laser.Paint(dc);
  electron.Paint(dc);
  CVector pl,pe;
  terminator->GetPoints(pl,pe);
  pe=matrix*(pe-trans);
  pl=matrix*(pl-trans);
  laser.SetPoint((int)pl[0]+dx,(int)pl[1]+dy);
  laser.Paint(dc);
  electron.SetPoint((int)pe[0]+dx,(int)pe[1]+dy);
  electron.Paint(dc);
}

